#define SRC_FILE    1
#define DEST_FILE   2
#define NFO_FILE    3
#define REPEATS     4
#define DELAY       5
#define DECAY       6
#define SATUR_COEF  7
#define FATHER_PID  8
#define NB_OF_ARGS  9

#include<stdio.h>
#include<fcntl.h>
#include<unistd.h>
#include<math.h>
#include<stdlib.h>
#include<signal.h>
#include"DebugLib.h"
#include "Filter_Data.h"
#include "Fct_Sys.h"

#undef BUFFER_SIZE
#define BUFFER_SIZE 65536*10

int main(int argc, char* argv[])
{
  /* Arguments pour le plug-in:

     - fichier source
     - fichier destination
     - fichier info contenant un enregistrement
     de type t_filter_data

     Agruments pour ce programme:

     - fichier source
     - fichier destination
     - nombre de rptitions
     - delay entre chaque rptition
     - baisse de volume entre chaque rptition

    Codes d'erreur retourns par le plug-in:
    -1: nombre d'arguments incorrects
    -2: erreur lors de l'ouverture du source
    -3: erreur lors de l'ouverture de la destination
  */
  /*
    #define ECHOES_ERRORS 
    ("Internal error - bad number of parameters",
    "Error while openning source file",
    "Error while openning destination file",
    "*/

  int src_file, dest_file;
  short int src_buf[BUFFER_SIZE];
  short int dest_buf[BUFFER_SIZE/2];
  size_t read_result;
  int repeats, delay;
  float decay, decay_coef,saturation_coef;
  int return_value,father_pid;
  t_filterdata* filterdata;
  size_t orig_size;

  // PID_T pid;
  size_t pos,i,k;
  float coef;
  
  return_value = 0;
  if (argc != NB_OF_ARGS)
    {
      printf("echoes.c: main: Incorrect number of arguments (here %d, "
	     "needed %d) - Syntax:\n"
	     "./echoes source_file dest_file info_file nb_repeat delay "
	     "decay saturation_coefficient father_pid\n",argc-1, NB_OF_ARGS-1);
      
      /*      for (i=1;i<argc;i++)
	      {
	      printf("Argument %d: %s\n",i,argv[i]);
	      }
      */
      if (father_pid !=0)
	kill(father_pid,15);
      
      return(-1);
    }
  
  
  repeats         = atoi(argv[REPEATS]);
  delay           = atoi(argv[DELAY]);
  decay_coef      = (float)atoi(argv[DECAY]) / 100;    // in purcent
  saturation_coef = (float)atoi(argv[SATUR_COEF]) / 100; // in purcent
  father_pid      = atoi(argv[FATHER_PID]);
  
  src_file = open(argv[SRC_FILE], O_RDWR);
  orig_size = give_file_size(src_file);
  ftruncate(src_file, orig_size + repeats * delay);

  filterdata     = filterdata_new(argv[NFO_FILE]);
  
  if (src_file<0)
    {
      printf("echoes.c: main: error while openning '%s', source file\n",argv[SRC_FILE]);
      if (father_pid !=0)
      	kill(father_pid,15);
      
      return(-2);
    }
  
  
  dest_file = open(argv[DEST_FILE], O_WRONLY | O_CREAT, 0644);
  
  if (dest_file<0)
    {
      printf("echoes.c: main: error while openning '%s', destination file\n",argv[DEST_FILE]);
      if (father_pid !=0)
      	kill(father_pid,15);
      
      return(-3);
    }
  
  for(i=0; i<(BUFFER_SIZE / 2); i++)
    {
      src_buf[i] = 0;
    }
  
  read_result = read(src_file, src_buf+BUFFER_SIZE/2, BUFFER_SIZE); // on lit 1/2 buffer au dbut.
  read_result *= 2;
  
  while (read_result != 0)
    {
      decay = 1;
      
      for (k = 0; k <= repeats; k++)
	{
	  decay = pow(decay_coef,k);
	  i = 0;
	  if (BUFFER_SIZE/2 - delay * k <0)
	    {
	      pos = BUFFER_SIZE/2;
	      warning("echoes.c: main: delay is higher than buffer size");
	      return_value = -1;
	    }
	  else
	    pos = BUFFER_SIZE/2 - delay*k;
	  
	  coef = decay * saturation_coef;
	  while (i < read_result / 4)
	    {
	      if (k==0) 
		{
		  dest_buf[i] = (short int)((float)src_buf[i+pos] * coef);
		}
	      else
		{
		  dest_buf[i] += (short int)((float)src_buf[i + pos] * coef);
		}
	      i++;
	    }
	}
      write(dest_file, dest_buf, read_result/2);
      lseek(src_file, -(read_result/2), SEEK_CUR);
      if (read_result == BUFFER_SIZE*2) 
	{
	  read_result = read(src_file, src_buf, BUFFER_SIZE*2);
	  for (k=read_result/2+1; k<BUFFER_SIZE; k++)
	    src_buf[k]=0;
	}
      else
	break;
      
    }
  ftruncate(src_file, orig_size);
  close(src_file);
  close(dest_file);
  
  if (father_pid !=0)
    kill(father_pid,15);
  filterdata_destroy(filterdata);
  return(return_value);
}

